// TestWeakPtr.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <IOStream>
#include <Windows.h>
#include "boost\shared_ptr.hpp"
#include "boost\weak_ptr.hpp"


static boost::shared_ptr<int> g_pInt;
static boost::weak_ptr<int> g_pWeakInt;


unsigned long TestWeakPtr(bool i_isMainThread=false)
{
	for (int i(0); i<100000000; ++i)
	{
		int nbPtr = g_pInt.use_count();

		// since g_pInt is static, we should always have at least 1 count
		if (nbPtr < 1)
		{
			printf("%i:%i - Bug count %i [1]\n", (i_isMainThread) ? 1:0, i, nbPtr);			
		}

		boost::shared_ptr<int> pIntTemp(g_pWeakInt);

		nbPtr = g_pInt.use_count();

		// since we took a copy of the global static weak_ptr into a 
		// shared_ptr, the count of the global shared_ptr should be 
		// incremented and at least 2
		if (nbPtr < 2)
		{
			printf("%i:%i - Bug count %i [2]\n", (i_isMainThread) ? 1:0, i, nbPtr);		
		}

	}

	return 0;
}


int main(int /*argc*/, char* /*argv*/[])
{
	g_pInt.reset(new int(5));
	g_pWeakInt = g_pInt;

	unsigned long threadID(0);
	HANDLE threadHdl =  CreateThread(
				NULL,
				  0,
				  (unsigned long (__stdcall *)(void *))&TestWeakPtr,
				  NULL,
				  0,
				  &threadID
				);

	TestWeakPtr(true);

	WaitForSingleObject(threadHdl, INFINITE);

	if (!g_pInt.unique())
	{
		printf("FINAL BUG: %i\n", g_pInt.use_count());
	}	

	g_pInt.reset();

	printf("Finished\n");
	getchar();

	return 0;
}
